/***************************************************************************/
/*                                                                         */
/* FILE NAME : MyGAfrGP.c                                                  */
/*                                                                         */
/*                                                                         */
/*  COLLECT_STAT_FOR_GA 
*/
/***************************************************************************/

/*
  Program for graph partitioning using GA's, written by Dr. Ching-Tai Chiu 
*/

#include "ga.h"
#include <stdio.h>

extern void generate_initial_set();
extern double generate_sigma_graph();
extern void generate_graph();
extern double calc_pop_fitness(), calc_fitness();
extern void reproduction();
extern void rescale();
int evol, spop, besti;
double zs[MAX_POP];        /* Fitness value of this solution */ 
double w[MAX_ND][MAX_ND];  /* Weights between graph nodes */
double wS[MAX_ND][MAX_ND]; /* Starting Weights between graph nodes */
double MainFitnessValue;
float p[MAX_POP];
int nodecount;
int bstr[MAX_POP][MAX_ND], bestsol[MAX_ND];
FILE *fp;

DebugPrint(str)
char *str;
{ /*printf("\n %s \n",str);
  */
  return;
}

/*
*  Normalize the weight values such that sum of weights is equal to 1.
*/
void Normalize(nodenum)
int nodenum;
{  int i,j;
   double total = 0.0;

   for(i=0;i<nodenum;i++) 
      for(j=0;j<nodenum;j++) total += w[i][j];

   if(total!=0)
   {  for(i=0;i<nodenum;i++)
        for(j=0;j<nodenum;j++) 
           w[i][j] = w[i][j] / total;
   } 
   else printf("\n In Normalize() total is ZERO \n");
      
}/* end of Normalize() */

/*
* Finds the minimum valued element's index from the double array.
*/
int Find_minimum(darray,size)
double darray[];
int size;
{  int i;
   int min = 0;
   for(i=1;i<size;i++)
      if(darray[min]>darray[i]) min=i;
   return min;
}/* Find_minimum() */

/*
* Prints the population information. Each solution. Each solution's
* fitness value and if it has the minimum fitness value then this solution
* has a double star.
*/
PrintPopulation(fp, gen_num, nodecount, pop_size)
FILE *fp;
int gen_num;
int nodecount;
int pop_size;
{  int i,j; 
   int min;

   min=Find_minimum(zs,pop_size);

   if(gen_num>=0) fprintf(fp,"\n Generation : %d ",gen_num);

   for(i=0;i<pop_size;i++)
   {  fprintf(fp,"\n Solution %3d -->",i); 
      for(j=0;j<nodecount;j++) fprintf(fp, "%d", bstr[i][j]);
      fprintf(fp,"  %8.4f ",zs[i]);
      if((i==min)||(zs[i]==zs[min])) fprintf(fp," ** ");
   }
   fprintf(fp,"\n");
}/* end of PrintPopulation() */

/*
* Shows the population information. Each solution. Each solution's
* fitness value and if it has the minimum fitness value then this solution
* has a double star.
*/
ShowPopulation(gen_num, nodecount, pop_size)
int gen_num;
int nodecount;
int pop_size;
{  int i,j; 
   int min;

   min=Find_minimum(zs,pop_size);

   if(gen_num>=0) printf("\n Generation : %d ",gen_num);

   for(i=0;i<pop_size;i++)
   {  printf("\n Solution %3d -->",i); 
      for(j=0;j<nodecount;j++) printf("%d", bstr[i][j]);
      printf("  %8.4f ",zs[i]);
      if((i==min)||(zs[i]==zs[min])) printf(" ** ");
   }
   printf("\n");
   getchar();
}/* end of PrintPopulation() */

solveProblemOnce(cross_prob, mut_prob, evol, nodecount, spop , fp)
double cross_prob;
double mut_prob;
int evol;
int nodecount;
int spop;
FILE *fp;
{
  int i, j, worsti, count0, count1;
  double avr_fit, zmax;
  long int clock;
  char mfx[30], mfy[30];

  DebugPrint(" I generated GRAPH "); 

  clock = time(0);
  generate_initial_set();   /* Randomly find the weight values */
  ShowPopulation(0, nodecount, spop);
  /*Normalize(nodecount);     Normalize the weight values */
  avr_fit = calc_pop_fitness();  /* Calculate the population's fitness */
  rescale();

  /*DebugPrint(" Let's print out what we got so far "); 
  */

  if(fp!=NULL)
  for (i=0; i<spop; i++) {
    fprintf(fp, "\n%d    %6.2f  %6.3f --- ", i, zs[i], p[i]);
    for (j=0; j<nodecount; j++) fprintf(fp, "%d", bstr[i][j]);
  }

  if(fp!=NULL)
  { fprintf(fp, "\nAverage fitness = %8.4f    Min[%d]=%6.3f\n", 
	  avr_fit, besti, zs[besti]);

    fprintf(fp, " %d\n", 0);
    fprintf(fp, " %5.0f\n", zs[besti]);
  }

  /* HERE main generation starts. */

 printf("Generation : ");

  for (i=0; i<evol; i++) 
  {
    printf(" %d ",i);

    /*
    PrintPopulation(fp, i, nodecount, spop);
    */

    reproduction();

/*    zmax = zs[0];
    worsti = 0;
    for (j=0; j<spop; j++) 
      if (zmax < zs[j]) {
	worsti = j;
	zmax = zs[j];
      }
    for (j=0; j<nodecount; j++)	bstr[worsti][j] = bestsol[j];
*/
    avr_fit = calc_pop_fitness();

    /*
    printf("Average fitness at %d is %8.3f ",i,avr_fit);
    DebugPrint("EVOL2");
    */

    rescale();

    /*
    DebugPrint("EVOL3");

    printf("\nGeneration %4d: average fitness = %8.4f    Min[%d]=%6.3f\n", 
	   i+1, avr_fit, besti, zs[besti]);
    */

    /*
    fprintf(fp, " %d\n", i+1);
    fprintf(fp, " %5.0f\n", zs[besti]);

    if ((i % 10) == 0)
      fprintf(fp, "\n%5d: Average fitness = %8.4f  Min[%d]=%6.3f", 
	      i+1, avr_fit, besti, zs[besti]);   
    */


    /*
    if (drand48()<0.02 || (avr_fit - zs[besti]) < 0.005) {
    */

    if (myRandom(0.0,1.0)<mut_prob || (avr_fit - zs[besti]) < 0.005) {

      mutation();

      avr_fit = calc_pop_fitness();

      rescale();

/*
      fprintf(fp, "\n%5d: Average fitness = %8.4f  Min[%d]=%6.3f", 
	      i+1, avr_fit, besti, zs[besti]);
      fprintf(fp, " %d\n", i+1);
      fprintf(fp, " %5.0f\n", zs[besti]);
    */ 

      /*
      printf("\n%5d: Average fitness = %8.4f  Min[%d]=%6.3f", 
	      i+1, avr_fit, besti, zs[besti]);
      printf( " %d\n", i+1);
      printf( " %5.0f\n", zs[besti]);
      */


/*      zmax = zs[0];
      worsti = 0;
      for (j=0; j<spop; j++) 
	if (zmax < zs[j]) {
	  worsti = j;
	  zmax = zs[j];
	}
      for (j=0; j<nodecount; j++) bstr[worsti][j] = bestsol[j];  */


    }
  }


  if(fp!=NULL)
  {
  clock = time(0) - clock;
  for (i=0; i<spop; i++) {
    fprintf(fp, "\n%d    %6.2f  %6.3f --- ", i, zs[i], p[i]);
    for (j=0; j<nodecount; j++) fprintf(fp, "%d", bstr[i][j]);
  }

  fprintf(fp, "\nNodes = %d\nPopulation = %d\nGenerations = %d",
	  nodecount, spop, evol);

  fprintf(fp, "\n\nBest cost = %8.2f -> ", calc_fitness(bstr[besti]));
  count0 = count1 = 0;
  for (i=0; i<nodecount; i++) 
    {
      fprintf(fp, "%d", bstr[besti][i]);
      if (bstr[besti][i] == 0)
	count0++;
      else
	count1++;
    }
  fprintf(fp, "\n\n%d nodes in 0 set\n%d nodes in 1 set\n", count0, count1);

  fprintf(fp, "\nTime = %d\n", clock);
  }
   
  printf("   ---> END \n");

}

/*
*
*/

double MeanOfPopulation(popsize)
int popsize;
{  double ttl = 0.0;
   int i;
   for(i=0;i<popsize;i++)
       ttl = ttl + zs[i];
   return ( ttl / popsize );
}/* MeanOfPopulation() */

/*
*
*/
double VarianceOfPopulation(popsize,mean)
int popsize;
double mean;
{  double ttl=0.0;
   int i;
   for(i=0;i<popsize;i++)
      ttl = ttl + square((zs[i]-mean));
   return ( ttl );
}/* VarianceOfPopulation() */

/*
*
*/
double solveOneProblem(cross_prob, mut_prob, evol, nodecount, spop , fp, NumOfTimes,StatFP)
double cross_prob;
double mut_prob;
int evol;
int nodecount;
int spop;
FILE *fp;
int NumOfTimes;
FILE *StatFP;
{
  int i, j, bidx;
  double avr_fit, zmax;
  long int clock;
  char mfx[30], mfy[30];
  double TheBestVariance, TheBestMean, TheBestFittness;

  srand48(150*2);

  solveProblemOnce(cross_prob, mut_prob,evol, nodecount, spop , fp); 
  TheBestFittness = zs[Find_minimum(zs,spop)];
  TheBestMean=MeanOfPopulation(spop);
  TheBestVariance=VarianceOfPopulation(spop,TheBestMean);

  for(i=1;i<NumOfTimes;i++)
  {  
      srand48(150*(i+2));
      solveProblemOnce(cross_prob, mut_prob, evol, nodecount, spop , fp); 
      bidx = Find_minimum(zs,spop);
      if(zs[bidx]<TheBestFittness)
      {   TheBestFittness = zs[bidx];
	  for(j=0;j<nodecount;j++) bestsol[j] = bstr[bidx][j];
          TheBestMean = MeanOfPopulation(spop);
          TheBestVariance = VarianceOfPopulation(spop,TheBestMean);
      }/* end of if */

  }/* end of for */

  /* Produce report */
  fprintf(fp,"\n The Solution :");
  for(i=0;i<nodecount;i++) fprintf(fp,"%d",bestsol[i]);
  fprintf(fp,"\n");
  fprintf(fp,"\n The Fittness : %f ",TheBestFittness);
  fprintf(fp,"\n The Mean of the population : %f ",TheBestMean);
  fprintf(fp,"\n The Variance of the population : %f \n",TheBestVariance);

  fprintf(StatFP,"%5d %12.6f %12.6f %12.6f \n",evol,TheBestFittness,TheBestMean,TheBestVariance);

  return ( TheBestFittness );

}/* solveOneProblem() */

Show_Graph(nodecount,spop)
int nodecount;
int spop;
{  int k,m;

   /* prints the first spop-1 rows of the weight matrix */
   for(k=0;k<(nodecount-1);k++)
   {   for(m=0;m<nodecount;m++) printf("%9.4f ",w[k][m]);
       printf("\n");
   }/* end of for */

   /* prints the last row of the weight matrix */
   for(m=0;m<nodecount;m++) printf("%9.4f ",w[k][m]);

   /* prints the fitness value of this problem
   printf("%9.4f \n",zs[Find_minimum(zs,spop)]); */

}/* Show_Graph() */

/*
*
*
*/
read_weights(fp,k)
FILE *fp;
int k;
{  int i,j;
   int EndOfFile = 1;
   float fbuf;
   i=j=0;
   while((i<k)&&(EndOfFile))
   {  j=0;
      while((j<k)&&(EndOfFile))
      { if(j<(k-1))
        { if(fscanf(fp,"%f ",&fbuf)==EOF) EndOfFile = 0;
          else w[i][j] = fbuf;
        }
        else
         { if(fscanf(fp,"%f \n",&fbuf)==EOF) EndOfFile = 0;
           else w[i][j] = fbuf;
         }
        j++;
      }
      i++;
   }
   printf("\n read_weights i = %d \n",i);
   Show_Graph(k,spop);
   return(EndOfFile);
}/* read_weights() */


Print_Graph(nodecount,fp)
int nodecount;
FILE *fp;
{  int k,m;

   /* prints the first spop-1 rows of the weight matrix */
   for(k=0;k<(spop-1);k++)
   {   for(m=0;m<nodecount;m++) fprintf(fp,"%9.4f ",w[k][m]);
       fprintf(fp,"\n");
   }/* end of for */

   /* prints the last row of the weight matrix */
   for(m=0;m<nodecount;m++) fprintf(fp,"%9.4f ",w[k][m]);

   /* prints the fitness value of this problem */
   fprintf(fp,"%9.4f \n",zs[Find_minimum(zs,spop)]);

}/* Print_Graph() */

       
domain(outFile, NumberOfProblems,flag,QuesFileName,MinNumGen,MaxNumGen,NumberOfTry,cross_prob, mut_prob)
char *outFile;
int NumberOfProblems;
int flag;
char *QuesFileName;
int MinNumGen;
int MaxNumGen;
int NumberOfTry;
double cross_prob;
double mut_prob;
{
  int i, j, worsti, count0, count1;
  double avr_fit, zmax;
  long int clock;
  FILE *mpx, *mpy;
  char mfx[30], mfy[30];
  char GrFileName[30];
  int k, m;
  FILE *fpResults;
  FILE *fpStatistics;
  FILE *QuesFile;
  FILE *StatGA;
  double VarFitness,VarWeights;
  double diff;
  double FValue;

/*
  if (argc==1)
    {
      printf("Usage:\n");
      printf("num_nodes population epochs outputfile\n");
      exit(0);
    }
  sscanf(argv[1],"%d", &nodecount);
  sscanf(argv[2], "%d", &spop);
  sscanf(argv[3], "%d", &evol);

  if ((fp = fopen(argv[4], "w+t")) == NULL)
    {
      printf("Could not open output file\n");
      exit(0);
    }
*/

  if ((StatGA = fopen("GASTAT.tex", "w+t")) == NULL)
    {
      printf("Could not open output file\n");
      return;
    }

  if ((fp = fopen(outFile, "w+t")) == NULL)
    {
      printf("Could not open output file\n");
      return;
    }

  if ((fpStatistics = fopen("Stat", "w")) == NULL)
    {
      printf("Could not open output file\n");
      exit(0);
    }


  if ((fpResults = fopen("Results", "w")) == NULL)
    {
      printf("Could not open output file\n");
      exit(0);
    }
  DebugPrint("I opened the output file ");

  rewind(fp);

  /*
  strcpy(mfx, argv[4]);
  strcpy(mfy, argv[4]);
  */
  strcpy(mfx, "x.mat");
  strcpy(mfy, "y.mat");
  mpx = fopen(mfx, "w+t");
  mpy = fopen(mfy, "w+t");

  /* Write the number of nodes in the graph */

  fprintf(fpStatistics,"%d \n",nodecount);

  sprintf(GrFileName,"Graph%d",i);

  if(flag)
  {  if((QuesFile=fopen(QuesFileName,"r"))!=NULL) 
     {  read_weights(QuesFile,nodecount); 
	printf("\n Read is finished \n");
	fclose(QuesFile);
     }
     else generate_graph_write(GrFileName);
  }
  else generate_graph_write(GrFileName);

  /*for(i=0;i<NumberOfProblems;i++) */

  /* 
     Run the same problem with different generation numbers such as from
     nodecount to 20 * nodecount .
     Each solution is start from the new random population. 
     Program solves the problem 4 times and picks the best population
     according to the best fitness value of the individual.
  */

  for(i=MinNumGen;i<MaxNumGen;i++)
  {
     printf("\n Problem %d \n",i);

     FValue = solveOneProblem(cross_prob, mut_prob, i, nodecount, spop , fp, NumberOfTry, StatGA);

     /*printf("\n After SolveOneProblem \n");*/

     Print_Graph(nodecount,fpStatistics);

     /* prints the fitness value of this problem */
     fprintf(fp,"%9.4f \n",FValue);

     /*printf("\n After Print_Graph \n"); */

  }/* end of for */

  fclose(mpx);
  fclose(mpy);
  fclose(fp);
  fclose(fpStatistics);
  fclose(fpResults);
  fclose(StatGA);

}/* end of domain() */

IsStatNecessary(current,Max,NSpnts,PointList)
int current;
int Max;
int NSpnts;
int *PointList;
{  int i;
   int ret=0;
   if( current!=evol )
   {  for(i=0;i<NSpnts&&!ret;i++)
      {  if(current==PointList[i]) ret=1;
      }/* end of for */
   }/* end of if */
   return ret;
}/* IsStatNecessary() */

doStatisticsOfPopulation(Best,Mean,Variance,Min,Max)
double *Best,*Mean,*Variance,*Min,*Max;
{ int i,j;
  *Best = zs[Find_minimum(zs,spop)];
  *Mean=MeanOfPopulation(spop);
  *Variance=VarianceOfPopulation(spop,*Mean);
  *Min=*Max=zs[0];

  for(i=0;i<spop;i++)
    if(*Min>zs[i]) *Min=zs[i];
	else if(*Max<zs[i]) *Max=zs[i];
}/* doStatisticsOfPopulation() */

writeStatistics(fp)
FILE *fp;
{  double Best,Mean,Variance,Min,Max;  
   doStatisticsOfPopulation(&Best,&Mean,&Variance,&Min,&Max);

   fprintf(fp,"%12.7f %12.7f %12.7f ",Best,Min,Max);
   fprintf(fp,"%12.7f %12.7f \n",Mean,Variance);

}/* writeStatistics() */


solveProblemOnce2(cross_prob, mut_prob, evol, nodecount, spop , fp, NSpnts,
PointList,Trial,rdn,AppNum)
double cross_prob;
double mut_prob;
int evol;
int nodecount;
int spop;
FILE *fp;
int NSpnts;
int *PointList;
int Trial;
double rdn;
int AppNum;
{
  int i, j, worsti, count0, count1;
  double avr_fit, zmax;
  long int clock;
  char mfx[30], mfy[30];

  DebugPrint(" I generated GRAPH "); 

  clock = time(0);

  generate_initial_set();

  ShowPopulation(0, nodecount, spop);

  avr_fit = calc_pop_fitness();  /* Calculate the population's fitness */


  rescale();

  /* HERE main generation starts. */

  printf("Generation : ");

  for (i=0; i<evol; i++) 
  {
    /*printf(" %d Average fitness : %12.8f ",i,avr_fit); */

    if(IsStatNecessary(i,evol,NSpnts,PointList))
    {
       fprintf(fp,"%12.7f %12.7f ",(double)Trial,rdn);
       fprintf(fp,"%12.7f %12.7f ",(double)AppNum,(double)i);
       writeStatistics(fp);
       printf("\n I wrote statistics %d",i);

    }/* end of if */

    /* PrintPopulation(fp, i, nodecount, spop); */

    reproduction();

    avr_fit = calc_pop_fitness();

    rescale();

    if (myRandom(0.0,1.0)<mut_prob || (avr_fit - zs[besti]) < 0.005) 
    {
       mutation();

       avr_fit = calc_pop_fitness();

       rescale();

    }/* end of if */

    ShowPopulation(i, nodecount, spop);

  }/* end of for */

  printf("   ---> END \n");

}/* solveProblemOnce2() */

/*
*
*/
double solveOneProblem2(cross_prob, mut_prob, evol, nodecount, spop , NumOfTimes,StatFP, NSPnts, PointList,Trial,rnd,AppNum)
double cross_prob;
double mut_prob;
int evol;
int nodecount;
int spop;
int NumOfTimes;
FILE *StatFP;
int NSPnts;
int *PointList;
int Trial;
double rnd;
int AppNum;
{
  int i, j, bidx;
  double avr_fit, zmax;
  long int clock;
  char mfx[30], mfy[30];
  double TheBestVariance, TheBestMean, TheBestFittness;
  double MinFit, MaxFit;

  srand48(150*2);

  solveProblemOnce2(cross_prob, mut_prob, evol, nodecount, spop , StatFP, 
		    NSPnts, PointList,Trial,rnd,AppNum);

  TheBestFittness = zs[Find_minimum(zs,spop)];
  TheBestMean=MeanOfPopulation(spop);
  TheBestVariance=VarianceOfPopulation(spop,TheBestMean);
  MinFit=MaxFit=zs[0];

  for(i=0;i<spop;i++)
    if(MinFit>zs[i]) MinFit=zs[i];
    else if(MaxFit<zs[i]) MaxFit=zs[i];

  for(i=1;i<NumOfTimes;i++)
  {  
      srand48(150*(i+2));
      solveProblemOnce2(cross_prob, mut_prob, evol, nodecount, spop , StatFP, 
		    NSPnts, PointList,Trial,rnd,AppNum);
      bidx = Find_minimum(zs,spop);
      if(zs[bidx]<TheBestFittness)
      {   TheBestFittness = zs[bidx];
	  for(j=0;j<nodecount;j++) bestsol[j] = bstr[bidx][j];
          TheBestMean = MeanOfPopulation(spop);
          TheBestVariance = VarianceOfPopulation(spop,TheBestMean);
          for(i=0;i<spop;i++)
            if(MinFit>zs[i]) MinFit=zs[i];
            else if(MaxFit<zs[i]) MaxFit=zs[i];

      }/* end of if */

  }/* end of for */

  /* Write to the statistics file */

  fprintf(StatFP,"%12.7f %12.7f ",(double)Trial,rnd);
  fprintf(StatFP,"%12.7f %12.7f ",(double)AppNum,(double)evol);
  fprintf(StatFP,"%12.7f %12.7f %12.7f ",TheBestFittness,MinFit,MaxFit);
  fprintf(StatFP,"%12.7f %12.7f \n",TheBestMean,TheBestVariance);

  return ( TheBestFittness );

}/* solveOneProblem() */

int doesNotExist(StatFileName)
char *StatFileName;
{  int ret=1;
   FILE *fp;

   if((fp=fopen(StatFileName,"r"))!=NULL) 
   {  ret=0; fclose(fp); }

   return ret;

}/* doesNotExist() */

void Collect_Stat_For_GA(N,sigmastep,sigmainit,sigmalast,NumOfTrials,NumOfGAApp,PopSize,MaxGenNumber,samplepoints,NumOfSampPoints,cross_prob,mut_prob)
int N; /* Problem Size, i.e. number of nodes in the Graph */
double sigmastep;
double sigmainit;
double sigmalast;
int NumOfTrials;
int NumOfGAApp;
int PopSize;
int MaxGenNumber;
int *samplepoints;
int NumOfSampPoints;
double cross_prob;
double mut_prob;
{  double sigma=sigmainit;
   int i,j;
   FILE *fp;
   char StatFileName[30];
   double rnd;

   nodecount = N;
   spop = PopSize;

   for( sigma=sigmainit ; sigma<sigmalast ; sigma+=sigmastep )
   {  
      sprintf(StatFileName,"%s_%08.5f.out","Sigma",sigma); 

      if(doesNotExist(StatFileName))
      {
	 if((fp=fopen(StatFileName,"w"))!=NULL)
         {
            for(i=0; i<NumOfTrials ; i++)
            {  
	       rnd=generate_sigma_graph(N,sigma);

	       Show_Graph(N,PopSize);

	       /*printf("\n random value = %f",rnd);
	       */

	       for(j=0; j<NumOfGAApp ; j++)
	       {  
	           printf("\n Problem #: %d GA Application # = %d",i,j);
                   solveOneProblem2(cross_prob, mut_prob,MaxGenNumber,N,
				    PopSize,1 ,fp,NumOfSampPoints,samplepoints,
				    i,rnd,j);

	       }/* end of for */

            }/* end of for */

	    fclose(fp);

	 }/* end of if */

      } else printf("\n you might want to keep %s \n",StatFileName);

   }/* end of for */

}/* Collect_Stat_For_GA() */

screen(NumberOfNodes,PopSize,MinNumGen,MaxNumGen,NumOfTry,RepFName,InFName,cross_prob, mut_prob,NumOfProbPerSigma,InitialSigma,TermSigma,StepSigma,NumOfSampPoints,SampPointList)
int *NumberOfNodes;
int *PopSize;
int *MinNumGen;
int *MaxNumGen;
int *NumOfTry;
char *RepFName;
char *InFName;
double *cross_prob;
double *mut_prob;
int *NumOfProbPerSigma;
double *InitialSigma;
double *TermSigma;
double *StepSigma;
int *NumOfSampPoints;
int *SampPointList;
{ float a;
  int n,i;
  printf("\n Number of nodes :"); scanf("%d",NumberOfNodes);
  printf(" Size of the population :"); scanf("%d",PopSize);
  printf(" Cross over probability :"); if(scanf("%f",&a)) *cross_prob=(double)a;
  printf(" Mutation probability :"); if(scanf("%f",&a)) *mut_prob=(double)a;
  printf(" Minimum number of Generation :"); scanf("%d",MinNumGen);
  printf(" Maximum number of Generation :"); scanf("%d",MaxNumGen);
  printf(" How many times do you want me to run the GA for the problem :"); 
  scanf("%d",NumOfTry);

  printf(" How many times do you want me to produce problem for the same sigma:"); 
  scanf("%d",NumOfProbPerSigma);
  printf(" Initial sigma value :"); 
  if(scanf("%f",&a)) *InitialSigma=(double)a;
  printf(" Terminating Sigma value :"); 
  if(scanf("%f",&a)) *TermSigma=(double)a;
  printf(" Step size of sigma :"); 
  if(scanf("%f",&a)) *StepSigma=(double)a;
  printf(" Number of sampling points :");
  scanf("%d",NumOfSampPoints);
  i=0;
  while((i<*NumOfSampPoints)&&(i<10))
  {  printf(" Enter %d sampling point's generation number :",i+1);
     scanf("%d",&(SampPointList[i]));
     i++;
  }/* end of while */

  printf(" Name of the Output Report File :"); scanf("%s",RepFName);
  printf(" Name of the Weight file if it exists :"); scanf("%s",InFName);
  getchar();
}
       
main(argc, argv)
int argc;
char *argv[];
{
  char InFileName[30];
  int NumberOfProblems=40; /* Default */
  int a,b;
  int NumberOfNodes;
  int PopSize;
  int MinNumGen;
  int MaxNumGen;
  int NumOfTry;
  char RepFName[30];
  char InFName[30];
  int stay=1;
  char ch;
  double cross_prob = 0.5;
  double mut_prob = 0.02;
  int NumOfProbPerSigma;
  double InitialSigma;
  double TermSigma;
  double StepSigma;
  int NumOfSampPoints;
  int SampPointList[10];
  char buf[80];



  while(stay)
  { screen(&NumberOfNodes,&PopSize,&MinNumGen,&MaxNumGen,&NumOfTry,RepFName,InFName,&cross_prob,&mut_prob,&NumOfProbPerSigma,&InitialSigma,&TermSigma,&StepSigma,&NumOfSampPoints,SampPointList);

  printf("\n Number of Nodes : %d",NumberOfNodes);
  printf("\n Cross Over Prob : %f",cross_prob);
  printf("\n Mutation Prob   : %f",mut_prob);
  printf("\n Population Size : %d",PopSize);
  printf("\n Min. Gen. Num   : %d",MinNumGen);
  printf("\n Max. Gen. Num   : %d",MaxNumGen);
  printf("\n Number of Try   : %d",NumOfTry);
  printf("\n Number of problems per sigma   : %d",NumOfProbPerSigma);
  printf("\n Initial Sigma : %f",InitialSigma);
  printf("\n Terminating Sigma : %f",TermSigma);
  printf("\n Sigma Step : %f",StepSigma);
  printf("\n Number of sampling points : %d",NumOfSampPoints);

  printf("\n Sampling Points :");
  a=0;
  while(a<NumOfSampPoints&&a<10) { printf("%d ",SampPointList[a]); a++; }

  printf("\n Weight File Name: %s",InFName);

  printf("\n Do you want to continue? :"); 
  gets(buf);
  stay = (buf[0]=='N'||buf[0]=='n');

     if(stay)
     {  printf("\n Do you want to exit from the program? :"); 
	gets(buf);
        if(buf[0]=='y'||buf[0]=='Y') exit();
     }/* end of if */

  }/* end of while */

  nodecount = NumberOfNodes;
  spop = PopSize;
  evol = MaxNumGen;
  if(InFName[0]!=NULL)
  {  sprintf(InFileName,"%s",InFName);
     InFileName[29] = NULL;
     printf("\n I need to read from the %s \n",InFileName);
  }

  /* Produce Report */
  printf("\n Number of Nodes .....: %6d",nodecount);
  printf("\n Population Size .....: %6d",spop);
  printf("\n Maximum epoch number : %6d",evol);
  printf("\n Number of Problems ..: %6d \n",NumberOfProblems);

  Collect_Stat_For_GA(NumberOfNodes,StepSigma,InitialSigma,TermSigma,NumOfTry,NumOfProbPerSigma,PopSize,MaxNumGen,SampPointList,NumOfSampPoints,cross_prob,mut_prob);

/*
     domain(RepFName,NumberOfProblems,(InFName[0]!=NULL),InFileName,MinNumGen,MaxNumGen,NumOfTry,cross_prob, mut_prob);
*/

}/* end of main */
